@@ -53,7 +53,7 @@ impl LintRule for AdjacentOverloadSignatures {
 Overloaded signatures which are not next to each other can lead to code which is hard to read and maintain.
 
 ### Invalid:
-(bar is declared in-between foo overloads)
+(`bar` is declared in-between `foo` overloads)
 ```typescript
 type FooType = {
   foo(s: string): void;
@@ -85,7 +85,7 @@ export function bar(): void {}
 export function foo(sn: string | number): void {}
 ```
 ### Valid:
-(bar is declared after foo)
+(`bar` is declared after `foo`)
 ```typescript
 type FooType = {
   foo(s: string): void;
